#include "World.h"

// Include our actor class:
#include "SpinningTop.h"

World::World(Mesh groundMesh, Shader groundShader)
    : groundMesh(groundMesh)
    , groundShader(groundShader)
{
}

void World::render(Mat4f view, Mat4f projection){
    // At first, we want to simulate all the actors in the world:
    for(SpinningTop& spinningTop : spinningTops)
        spinningTop.simulate(*this);

    // Then, we render the ground mesh:
    {
        // Bind our shader:
        groundShader.bind();

        // Assume that our world mesh is already correctly transformed in the world
        // so we upload just the identity matrix, which is constructed by the default
        // constructor of Mat4f (see implementation):
        groundShader.setUniform("model", Mat4f());

        // Upload our view matrix, which transformates all point in the world so that
        // the viewing actor is in the origin:
        groundShader.setUniform("view", view);

        // Upload the projection matrix:
        groundShader.setUniform("projection", projection);

        // Draw our triangle:
        groundMesh.render();
    }

    // Finally, we want to render all the actors we simulated beforehand:
    for(SpinningTop& spinningTop : spinningTops)
        spinningTop.render(view, projection);
}
